# Instruction for Oracle database

**Note:** The following instruction describes installation for Oracle 21c.

You can find all the required files for the VLMatch user-defined extension (UDx) compilation in the following directory:

```
/var/lib/luna/current/extras/VLMatch/postgres/
```

For VLMatch UDx function compilation one needs to:

1. Install required environment, see [requirements](https://docs.oracle.com/en/database/oracle/oracle-database/21/lacli/installation-requirements-for-programming-environments-for-linux-x86-64.html):

```bash
sudo yum install gcc g++ 
``` 

2. Change `SDK_HOME` variable - oracle sdk root (default is `$ORACLE_HOME/bin`, check `$ORACLE_HOME` environment variable is set) in the makefile and run make from `base_scripts/database_matching/oracle` directory:

```bash
chmod +x make.sh
```

```bash
./make.sh
```

3. Define the library and the function inside the database (from database console):

```sql
CREATE OR REPLACE LIBRARY VLMatchSource AS '$ORACLE_HOME/bin/VLMatchSource.so';
CREATE OR REPLACE FUNCTION VLMatch(descriptorFst IN RAW, descriptorSnd IN RAW, length IN BINARY_INTEGER)
   RETURN BINARY_FLOAT 
AS
   LANGUAGE C
   LIBRARY VLMatchSource
   NAME "VLMatch"
   PARAMETERS (descriptorFst BY REFERENCE, descriptorSnd BY REFERENCE, length UNSIGNED SHORT, RETURN FLOAT);
```

4. Test function within call (from database console):

```sql
SELECT VLMatch(HEXTORAW('1234567890123456789012345678901234567890123456789012345678901234'), HEXTORAW('0123456789012345678901234567890123456789012345678901234567890123'), 32) FROM DUAL;
```

The result returned by the database must be "0.4765625".
