# Instruction for PostgreSQL database

**Note:** The following instruction describes installation for PostgreSQL 16.

## Standard matching option

You can find all the required files for the VLMatch user-defined extension (UDx) compilation in the following directory:

```
/var/lib/luna/current/extras/VLMatch/postgres/
```

For VLMatch UDx function compilation one needs to:

1. Install RPM repository:

```bash
dnf install -y https://download.postgresql.org/pub/repos/yum/reporpms/EL-8-x86_64/pgdg-redhat-repo-latest.noarch.rpm
```

2. Install PostgreSQL 16:

```bash
dnf install postgresql16-server
```

3. Install the development environment:

```bash
dnf install postgresql16-devel
```

4. Install the gcc-c++ package. The package version 4.8 or higher is required.

```bash
dnf install gcc-c++
```

5. Install CMAKE. The version 3.5 or higher is required.

6. Open the make.sh script using a text editor. It includes paths to the currently used PostgreSQL version. Change the following values (if necessary):

    * `SDK_HOME` specifies the path to PostgreSQL home directory. The default value is `/usr/pgsql-16/include/server`;
    * `LIB_ROOT` specifies the path to PostgreSQL library root directory. The default value is `/usr/pgsql-16/lib`.

7. Go to the make.sh script directory and run it:

```bash
cd /var/lib/luna/current/extras/VLMatch/postgres/ 
```

```bash
chmod +x make.sh
```

```bash
./make.sh
```

8. Define the function inside the service database:

```bash
sudo -u postgres -h 127.0.0.1 -- psql -d luna_events -c "CREATE FUNCTION VLMatch(bytea, bytea, int) RETURNS float8 AS 'VLMatchSource.so', 'VLMatch' LANGUAGE C PARALLEL SAFE;"
```

9. Test function by sending re following request to the service database:

```bash
sudo -u postgres -h 127.0.0.1 -- psql -d luna_events -c "SELECT VLMatch('\x1234567890123456789012345678901234567890123456789012345678901234'::bytea, '\x0123456789012345678901234567890123456789012345678901234567890123'::bytea, 32);"
```

The result returned by the database must be "0.4765625".

## Alternative matching options

When it is not possible to use postgres C-extensions for some reason, an alternative SQL user defined function can be used for matching.
Define the SQL-VLMatch function inside the service database:

```sql
CREATE OR REPLACE FUNCTION vlmatch(bytea, bytea, int) RETURNS float AS
-- original `vlmatch` function compatible interface
$BODY$
DECLARE
    acc float = 0;
BEGIN
    FOR i IN 0..$3-1 LOOP
        acc = acc + (get_byte($1, i) - get_byte($2, i)) ^ 2.0;
    END LOOP;
    RETURN acc / (128 * 128);
END;
$BODY$
LANGUAGE plpgsql PARALLEL SAFE IMMUTABLE;
```

Test the function by the following query to the database:

```sql
SELECT * FROM vlmatch('\x1234567890123456789012345678901234567890123456789012345678901234'::bytea, '\x0123456789012345678901234567890123456789012345678901234567890123'::bytea, 32)
```

The result returned by the database must be "0.4765625".

⚠️**Beware the performance penalty.**
When using the SQL-VlMatch, the query execution time is about two orders of magnitude slower than when using the standard C-VLMatch.
Also make sure your database has query parallelization configured.
